@@ -38,7 +38,6 @@ from mogo.field import Field, EmptyRequiredField
 from pymongo.dbref import DBRef
 from pymongo.objectid import ObjectId
 from mogo.decorators import notinstancemethod
-import inspect
 import logging
 
 class BiContextual(object):
@@ -50,8 +49,8 @@ class BiContextual(object):
     def __get__(self, obj, type=None):
         """ Return a properly named method. """
         if obj is None:
-            return getattr(type, "_class_"+self.name)
-        return getattr(obj, "_instance_"+self.name)
+            return getattr(type, "_class_" + self.name)
+        return getattr(obj, "_instance_" + self.name)
 
 class InvalidUpdateCall(Exception):
     """ Raised whenever update is called on a new model """
@@ -63,6 +62,36 @@ class UnknownField(Exception):
     """
     pass
 
+class NewModelClass(type):
+    """ Metaclass for inheriting field lists """
+
+    def __new__(cls, name, bases, attributes):
+        # Emptying fields by default
+        attributes["__fields"] = {}
+        new_model = super(NewModelClass, cls).__new__(cls, name,
+            bases, attributes)
+        new_model._update_fields() # pre-populate fields
+        return new_model
+
+    def __setattr__(cls, name, value):
+        """ """
+        super(NewModelClass, cls).__setattr__(name, value)
+        if isinstance(value, Field):
+            # Update the fields, because they have changed
+            cls._update_fields()
+
+
+class InvalidUpdateCall(Exception):
+    """ Raised whenever update is called on a new model """
+    pass
+
+class UnknownField(Exception):
+    """ Raised whenever an invalid field is accessed and the
+    AUTO_CREATE_FIELDS is False.
+    """
+    pass
+
+
 class NewModelClass(type):
     """ Metaclass for inheriting field lists """
 
@@ -78,7 +107,7 @@ class NewModelClass(type):
         return new_model
 
     def __setattr__(cls, name, value):
-        """ """
+        """ Catching new field additions to classes """
         super(NewModelClass, cls).__setattr__(name, value)
         if isinstance(value, Field):
             # Update the fields, because they have changed
@@ -115,18 +144,33 @@ class Model(dict):
         instance = cls(**kwargs)
         return instance
 
+    @classmethod
+    def use(cls, session):
+        """ Wraps the class to use a specific connection session """
+        class Wrapped(cls):
+            pass
+        Wrapped.__name__ = cls.__name__
+        connection = session.connection
+        collection = connection.get_collection(Wrapped._get_name())
+        Wrapped._collection = collection
+        return Wrapped
+
+    @classmethod
+    def create(cls, **kwargs):
+        """ Create a new model and save it. """
+        if hasattr(cls, "new"):
+            model = cls.new(**kwargs)
+        else:
+            model = cls(**kwargs)
+        model.save()
+        return model
+
     def __init__(self, **kwargs):
-        """ Just initializes the fields. This should ONLY be called
-        from .new() or the Cursor.
-        """
+        """ Creates an instance of the model, without saving it. """
         super(Model, self).__init__()
-        is_new_instance = "_id" not in kwargs
+        is_new_instance = self._id_field not in kwargs
         for field, value in kwargs.iteritems():
-            private_field = field.startswith("_")
-            if private_field:
-                # Setting it in the dict, but leaving it alone
-                self[field] = value
-            elif is_new_instance:
+            if is_new_instance:
                 if field in self._fields.values():
                     # Running validation, if the field exists
                     setattr(self, field, value)
@@ -146,7 +190,10 @@ class Model(dict):
             self._fields[attr.id] = field_name
 
             # set the default
-            if attr.default is not None and not self.has_key(field_name):
+            # Code smell here -- we should be telling the field to do
+            # this behavior rather than introspecting the fields attributes.
+            if hasattr(attr, "default") and attr.default is not None \
+                    and not self.has_key(field_name):
                 self[field_name] = attr._get_default()
 
     @property
@@ -159,8 +206,6 @@ class Model(dict):
         """ (Re)update the list of fields """
         cls.__fields = {}
         for attr_key in dir(cls):
-            if attr_key.startswith('_'):
-                continue
             attr = getattr(cls, attr_key)
             if not isinstance(attr, Field):
                 continue
@@ -212,16 +257,24 @@ class Model(dict):
         if "safe" in kwargs:
             pass_kwargs["safe"] = kwargs.pop("safe")
         body = {}
+        checks = []
         for key, value in kwargs.iteritems():
             if key in self._fields.values():
                 setattr(self, key, value)
             else:
                 logging.warning("No field for %s" % key)
                 self[key] = value
-            body[key] = self[key] # PyMongo value
-        self._check_required(*body.keys())
+            # Attribute names to check.
+            checks.append(key)
+            # Field names in collection.
+            field = getattr(self.__class__, key)
+            field_name = field._get_field_name(self)
+            body[field_name] = self[field_name] # PyMongo value
+        logging.debug("Checking fields (%s).", checks)
+        self._check_required(*checks)
         coll = self._get_collection()
-        return coll.update(spec, { "$set":  body }, **pass_kwargs)
+        logging.debug("Setting body (%s)", body)
+        return coll.update(spec, {"$set":  body}, **pass_kwargs)
 
     update = BiContextual("update")
 
@@ -232,8 +285,9 @@ class Model(dict):
         for field_name in field_names:
             # check that required attributes have been set before,
             # or are currently being set
-            if not self.has_key(field_name):
-                field = getattr(self.__class__, field_name)
+            field = getattr(self.__class__, field_name)
+            storage_name = field._get_field_name(self)
+            if not self.has_key(storage_name):
                 if field.required:
                     raise EmptyRequiredField("'%s' is required but empty"
                                              % field_name)
@@ -255,6 +309,15 @@ class Model(dict):
     @notinstancemethod
     def remove(cls, *args, **kwargs):
         """ Just a wrapper around the collection's remove. """
+        if not args:
+            # If you get this exception you are calling remove with no
+            # arguments or with only keyword arguments, which is not 
+            # supported (and would remove all entries in the current
+            # collection if it was.) If you really want to delete
+            # everything in a collection, pass an empty dictionary like
+            # Model.remove({})
+            raise ValueError(
+                'remove() requires a query when called with keyword arguments')
         coll = cls._get_collection()
         return coll.remove(*args, **kwargs)
 
@@ -276,12 +339,20 @@ class Model(dict):
         """
         return self._get_id()
 
+    _id = id # for nod
+
     @classmethod
     def find_one(cls, *args, **kwargs):
         """
         Just a wrapper for collection.find_one(). Uses all
         the same arguments.
         """
+        if kwargs and not args:
+            # If you get this exception you should probably be calling first,
+            # not find_one. If you really want find_one, pass an empty dict:
+            # Model.find_one({}, timeout=False)
+            raise ValueError(
+                'find_one() requires a query when called with keyword arguments')
         coll = cls._get_collection()
         result = coll.find_one(*args, **kwargs)
         if result:
@@ -294,6 +365,12 @@ class Model(dict):
         A wrapper for the pymongo cursor. Uses all the
         same arguments.
         """
+        if kwargs and not args:
+            # If you get this exception you should probably be calling search,
+            # not find. If you really want to call find, pass an empty dict:
+            # Model.find({}, timeout=False)
+            raise ValueError(
+                'find() requires a query when called with keyword arguments')
         return Cursor(cls, *args, **kwargs)
 
     @classmethod
@@ -314,9 +391,29 @@ class Model(dict):
         for key, value in kwargs.iteritems():
             if isinstance(value, Model):
                 value = value.get_ref()
+            field = getattr(cls, key)
+
+            # Try using custom field name in field.
+            if field._field_name:
+                key = field._field_name
+
             query[key] = value
         return cls.find(query)
 
+    @classmethod
+    def search_or_create(cls, **kwargs):
+        "search for an instance that matches kwargs or make one with __init__"
+        obj = cls.search( **kwargs ).first()
+        if obj:
+            return obj
+        return cls.create( **kwargs )
+
+    @classmethod
+    def first(cls, **kwargs):
+        """ Helper for returning Blah.search(foo=bar).first(). """
+        result = cls.search(**kwargs)
+        return result.first()
+
     @classmethod
     def grab(cls, object_id):
         """ A shortcut to retrieve one object by its id. """
@@ -402,9 +499,18 @@ class Model(dict):
         """ Returns a DBRef for an document. """
         return DBRef(self._get_name(), self._get_id())
 
-    def __repr__(self):
+    def __unicode__(self):
+        """ Returns string representation. Overwrite in custom models. """
         return "<MogoModel:%s id:%s>" % (self._get_name(), self._get_id())
 
+    def __repr__(self):
+        """ Just points to __unicode__ """
+        return self.__unicode__()
+
+    def __str__(self):
+        """ Just points to __unicode__ """
+        return self.__unicode__()
+
 
 class PolyModel(Model):
     """ A base class for inherited models """
